@@ -12,7 +12,7 @@ use Helios::ObjectDriver::DBI;
 use Helios::ConfigParam;
 use Helios::Error::ConfigError;
 
-our $VERSION = '2.811_3850';
+our $VERSION = '2.60';
 
 =head1 NAME
 
@@ -24,24 +24,24 @@ Helios::Config - base class for Helios configuration system
  # a particular conf file and host
  use Helios::Config;
  Helios::Config->init(
-     conf_file => '/path/to/helios.ini',
-     hostname  => 'host',
-     service   => 'ServiceName'
+     CONF_FILE => '/path/to/helios.ini',
+     HOSTNAME  => 'host',
+     SERVICE   => 'ServiceName'
  );
  my $config = Helios::Config->parseConfig();
  
  # if $HELIOS_INI env var is set and current host is used
  use Helios::Config;
- Helios::Config->init(service => 'ServiceName');
+ Helios::Config->init(SERVICE => 'ServiceName');
  my $config = Helios::Config->parseConfig();
  
  # same as above; parseConfig() will automatically call init()
  use Helios::Config;
- my $config = Helios::Config->parseConfig(service => 'ServiceName');
+ my $config = Helios::Config->parseConfig(SERVICE => 'ServiceName');
  
  # you can also use accessor methods; for example, with Helios::TestService:
  use Helios::Config;
- Helios::Config->setService('Helios::TestService');
+ Helios::Config->setServiceName('Helios::TestService');
  Helios::Config->init();
  my $config = Helios::Config->parseConfig();
 
@@ -50,7 +50,7 @@ Helios::Config - base class for Helios configuration system
  use Helios::Config;
  my $config;
  eval {
- 	Helios::Config->init(service => 'Helios::TestService');
+ 	Helios::Config->init(SERVICE => 'Helios::TestService');
  	$config = Helios::Config->parseConfig();
  } or do {
  	my $E = $@;
@@ -104,7 +104,7 @@ from data in the environment.
                           -defaults to $HELIOS_INI env variable
  set/getHostname()        hostname the Helios service is running on
                           -defaults to results of Sys::Hostname::hostname()
- set/getService()         name of the running Helios service
+ set/getServiceName()     name of the running Helios service
                           -defaults to undefined, which will cause the
                            resulting config to contain the contents of the 
                            helios.ini [global] section only
@@ -169,8 +169,7 @@ sub getServiceName {
     no strict 'refs';
     return $$var;	
 }
-sub setService { setServiceName(@_); }
-sub getService { getServiceName(@_); }
+
 
 my $Driver;
 sub setDriver {
@@ -272,13 +271,10 @@ set up Helios collective would only specify the service:
 sub init {
 	my $self = shift;
 	my %params = @_;
-	foreach (keys %params) {
-		$params{lc($_)} = $params{$_};
-	}
-	if ( defined($params{conf_file}) ) { $self->setConfFile($params{conf_file}); }
-	if ( defined($params{service})   ) { $self->setServiceName($params{service}); }
-	if ( defined($params{hostname})  ) { $self->setHostname($params{hostname}); }
-	if ( defined($params{debug})     ) { $self->debug($params{debug}); }
+	if ( defined($params{CONF_FILE}) ) { $self->setConfFile($params{CONF_FILE}); }
+	if ( defined($params{SERVICE})   ) { $self->setServiceName($params{SERVICE}); }
+	if ( defined($params{HOSTNAME})  ) { $self->setHostname($params{HOSTNAME}); }
+	if ( defined($params{DEBUG})     ) { $self->debug($params{DEBUG}); }
 	
 	# pull hostname from the environment if not already set
 	unless ( $self->getHostname() ) {
@@ -412,7 +408,7 @@ Given an optional conf file and an optional service name, parseConfFile()
 parses the conf file and returns the resulting hashref to the calling routine.  
 It also makes the hashref available via the getConfFileConfig() accessor.  
 If either the conf file or the service is not specified, the values from the 
-getConfFile() and/or getService() accessor(s) are used.  The conf file 
+getConfFile() and/or getServiceName accessor(s) are used.  The conf file 
 location is set by init() to the value of the $HELIOS_INI environment variable 
 unless otherwise specified.
 
@@ -473,11 +469,7 @@ sub parseConfFile {
 	unless (-r $conf_file) { Helios::Error::ConfigError->throw("Cannot read conf file $conf_file"); }
 	
 	my $cif = Config::IniFiles->new( -file => $conf_file );
-	unless ( defined($cif) ) { 
-		# @Config::IniFiles::errors contains the parse error(s);
-		my $E = join(" ", @Config::IniFiles::errors);
-		Helios::Error::ConfigError->throw("parseConfFile(): Invalid config file: $E"); 
-	}
+	unless ( defined($cif) ) { Helios::Error::ConfigError->throw("Invalid config file; check configuration"); }
 
 	# global must exist 
 	if ($cif->SectionExists("global") ) {
@@ -506,7 +498,7 @@ The parseConfDb() method queries the Helios collective database for
 configuration parameters matching the specified service name and hostname and 
 returns a hashref with those parameters to the calling routine.  If the service 
 name and hostname are not specified, the values returned from the 
-getService() and getHostname() accessors are used.  The getHostname() value 
+getServiceName() and getHostname() accessors are used.  The getHostname() value 
 is normally set by init() to the value returned by Sys::Hostname::hostname() 
 unless otherwise specified.
 
@@ -552,21 +544,17 @@ sub parseConfDb {
 	my @dbparams;
 
 	my $driver = $self->getDriver();
-	
-	@dbparams = $driver->search( 'Helios::ConfigParam' => {
-			worker_class => $service_name,
-			host => ['*', $self->getHostname() ],
-		}
-	);
-	foreach(@dbparams) {
+	@dbparams = $driver->search( 'Helios::ConfigParam' => { worker_class => $service_name, host => '*'} );
+	foreach (@dbparams) {
 		if ($self->debug) { print $_->param(),'=>',$_->value(),"\n"; }
-		if ( $_->host eq '*') {
-			$conf_all_hosts->{$_->param()} = $_->value();
-		} else {
-			$conf_this_host->{ $_->param() } = $_->value();
-		}
+		$conf_all_hosts->{$_->param()} = $_->value();
 	}
-	
+	@dbparams = $driver->search( 'Helios::ConfigParam' => { worker_class => $service_name, host => $hostname} );
+	foreach (@dbparams) {
+		if ($self->debug) { print $_->param(), '=>', $_->value(), "\n"; }
+		$conf_this_host->{ $_->param() } = $_->value();
+	}
+
 	$conf = $conf_all_hosts;
 	while ( my ($key, $value) = each %$conf_this_host) {
 		$conf->{$key} = $value;
@@ -576,240 +564,6 @@ sub parseConfDb {
 }
 
 
-
-
-
-=head2 getParam(param => $param_name [, service => $service_name] [, hostname => $hostname])
-
-Given a service name, parameter name, and (optionally) a hostname, getParam() 
-returns the parameter name's value to the calling routine.
-
-If hostname is not specified, the current host is assumed.
-
-=cut
-
-sub getParam {
-	my $self = shift;
-	my %params;
-	if (scalar @_ == 1) {
-		$params{param} = $_[0];
-	} else {
-		%params = @_;		
-	}
-	my $service_name = defined($params{service}) ? $params{service} : $self->getServiceName;
-	my $param_name   = $params{param};
-	my $host         = defined($params{hostname}) ? $params{hostname} : $self->getHostname();
-	my $conf = {};
-
-	# shortcut: if the current config hash has already been retrieved
-	# and the requested param is set, return *that* param
-	if ( defined($self->getConfig()) && 
-		defined($self->getService) && ($self->getService eq $service_name) &&
-		defined($self->getHostname) && ($self->getHostname eq $host) && 
-		defined($self->getConfig()->{$param_name}) 
-		) {
-		return $self->getConfig()->{$param_name};
-	}
-
-	# if we don't have everything, stop before we try
-	unless ($service_name && $host && $param_name) {
-		Helios::Error::ConfigError->throw('getParam(): service and param are required.');
-	}
-
-
-	eval {
-		my $driver = $self->getDriver();
-		my @dbparams = $driver->search( 'Helios::ConfigParam' => {
-				worker_class => $service_name,
-				param        => $param_name,
-				host => ['*', $host ],
-			}
-		);
-		my %conf_all_hosts;
-		my %conf_this_host;
-		foreach(@dbparams) {
-			if ($self->debug) { print $_->worker_class(),'|', $_->host(),'|', $_->param(),'=>',$_->value(),"\n"; }
-			if ( $_->host() eq '*') {
-				$conf_all_hosts{$_->param()} = $_->value();
-			} else {
-				$conf_this_host{ $_->param() } = $_->value();
-			}
-		}
-		$conf = \%conf_all_hosts;
-		
-		# if host=*, we're done
-		# otherwise, use the given host, if the param is available
-		if ( $host ne '*' && defined($conf_this_host{$param_name}) ) {
-			$conf->{$param_name} = $conf_this_host{$param_name};
-		}
-
-		1;
-	} or do {
-		my $E = $@;
-		Helios::Error::ConfigError->throw("getParam(): $E");
-	};
-
-	if ($self->debug && !defined($conf->{$param_name})) {
-		print "$service_name|$host|$param_name not found.\n";
-	}
-
-	return $conf->{$param_name};
-}
-
-# this is officially *undocumented*
-sub getAllParams {
-	my $self = shift;	
-	my $conf_all_hosts = {};
-	my $conf_this_host = {};
-	my $conf = {};
-
-	eval {
-		my $driver = $self->getDriver();	
-		my @dbparams = $driver->search( 'Helios::ConfigParam' );
-		foreach(@dbparams) {
-			if ($self->debug) { print $_->param(),'=>',$_->value(),"\n"; }
-			$conf->{ $_->worker_class() }->{ $_->host() }->{ $_->param() } = $_->value();
-		}
-		
-		1;
-	} or do {
-		my $E = $@;
-		Helios::Error::ConfigError->throw("getAllParams(): $E");
-	};
-
-	return $conf;		
-}
-
-
-=head2 setParam(param => $param_name [, service => $service_name]  [, hostname => $hostname], value => $value)
-
-Given a service name, parameter name, parameter value, and (optionally) 
-a hostname, setParam() sets the value for that parameter for that service 
-(and host) in the Helios collective database.  If hostname is not specified,
-the current host is assumed.  To set a parameter for all instances of a 
-service in a collective, set the hostname to '*'.
-
-=cut
-
-sub setParam {
-	my $self = shift;
-	my %params = @_;
-	my $service_name = defined($params{service}) ? $params{service} : $self->getServiceName;
-	my $param_name   = $params{param};  
-	my $host         = defined($params{hostname}) ? $params{hostname} : $self->getHostname;   
-	my $value        = $params{value};
-	my $cp;
-
-	# if we don't have everything, stop before we try
-	unless ($service_name && $host && $param_name && defined($value) ) {
-		Helios::Error::ConfigError->throw('setParam(): Service, param name, and value are required.');
-	}
-
-	eval {
-		# HELIOS_PARAMS_TB does not have a Primary Key.
-		# Because of that, we cannot use D::OD in the normal way (search() or 
-		# lookup(), change the value, then save()).
-		# If we find an existing param matching service|host|param, we have to
-		# delete ALL of the matching service|host|param in the table, then 
-		# create a new one.
-		# In SQL terms, we'll always do SELECT>DELETE>INSERT instead of 
-		# SELECT>UPDATE|INSERT.
-
-		# query for existing service/host/param
-		my $driver = $self->getDriver();	
-		my @cps = $driver->search( 'Helios::ConfigParam' => {
-				worker_class => $service_name,
-				param        => $param_name,
-				host         => $host,
-			}
-		);
-		my $cp = shift @cps;
-	
-		# ok, if there is a service/host/param exists,
-		# we have to clear it out first, then create a new one from scratch
-		if (defined($cp)) {
-			if ($self->debug) { print "$service_name|$host|$param_name already set to ",$cp->value,".  Clearing.\n"; }
-			$driver->remove('Helios::ConfigParam' =>
-				{
-					worker_class => $service_name,
-					host         => $host,
-					param        => $param_name
-				}, 
-				{ nofetch => 1 } 
-			);
-		} else {
-			if ($self->debug) { print "$service_name|$host|$param_name not found.  Creating.\n"; }
-		}
-
-		# now, create a new Helios::ConfigParam and insert into the database
-		if ($self->debug) { print "$service_name|$host|$param_name setting to $value\n"; }
-		$cp = Helios::ConfigParam->new();
-		$cp->worker_class($service_name);
-		$cp->host($host);
-		$cp->param($param_name);
-		$cp->value($value);	
-		$driver->insert($cp);
-		
-		1;
-	} or do {
-		my $E = $@;
-		# rethrow the error as a ConfigError
-		Helios::Error::ConfigError->throw("setParam(): $E");
-	};
-
-	return 1;
-}
-
-
-=head2 unsetParam(param => $param_name [, service => $service_name] [, hostname => $hostname,])
-
-Given a service name, parameter name, and (optionally) a hostname, unsetParam() 
-deletes that parameter's entry in the Helios collective database.  
-
-If hostname is not specified, the current host is assumed.  To unset a 
-parameter that is in effect for all instances of a service, you must set the 
-hostname to '*'.
-
-=cut
-
-sub unsetParam {
-	my $self = shift;
-	my %params = @_;
-	my $service_name = defined($params{service}) ? $params{service} : $self->getServiceName;
-	my $param_name   = $params{param};   
-	my $host         = defined($params{hostname}) ? $params{hostname} : $self->getHostname;
-	my $cp;
-
-	# if we don't have everything, stop before we try
-	unless ($service_name && $host && $param_name) {
-		Helios::Error::ConfigError->throw('setParam(): Service and param name are required.');
-	}
-
-	eval {
-		# delete ALL of the matching service|host|param in the table
-		my $driver = $self->getDriver();
-		if ($self->debug) { print "Clearing $service_name|$host|$param_name from param table.\n"; }
-		$driver->remove('Helios::ConfigParam' =>
-			{
-				worker_class => $service_name,
-				host         => $host,
-				param        => $param_name
-			}, 
-			{ nofetch => 1 } 
-		);
-		
-		1;
-	} or do {
-		my $E = $@;
-		# rethrow the error as a ConfigError
-		Helios::Error::ConfigError->throw("unsetParam(): $E");
-	};
-
-	return 1;
-}
-
-
-
 1;
 __END__
 
@@ -845,7 +599,7 @@ example:
  
  use MyConfig;
  
- sub ConfigClass { 'MyConfig' }
+ sub ConfigClass { 'MyConfig'; }
 
  sub run {
  	my $self = shift;
@@ -863,10 +617,343 @@ example:
 
 =head1 HELIOS CONFIGURATION PARAMETERS
 
-The Helios system itself defines a large number of configuration parameters to 
-control the helios.pl daemon, worker launching, and other system tasks.  
-Consult the L<Helios::Configuration> page for a full list of parameters and 
-their functions.
+The Helios system defines a large number of configuration parameters.  Some of 
+these affect the operation of the helios.pl daemon, others worker processes, 
+and some can affect both.  Aside from these reserved parameter names, the 
+configuration parameters your Helios service uses are largely up to you.  
+
+Helios configuration parameters that affect worker process launching and 
+management are usually in ALL CAPS.  This helps set them apart from other 
+application-level parameters.
+
+=head2 helios.pl
+
+=head3 Collective database configuration parameters
+
+These are the most important parameters in helios.ini.  They must be declared 
+in the [global] section.  Without them, helios.pl will be unable to connect to 
+the collective database and will fail to start.
+
+=head4 dsn
+
+ dsn=dbi:Oracle:SHARDEV
+
+The dsn parameter is the DBI datasource name of the Helios collective database.
+
+=head4 user
+
+ user=scott
+
+The user parameter is the user to use when connecting to the Helios 
+collective database.
+
+=head4 password
+
+ password=tiger
+
+The password parameter is the password to use when connecting to the Helios
+collective database.
+
+=head4 options
+
+ options=private_option=>'string',private_option2=>'another string'
+
+The options parameter is used when special DBI options are needed when 
+connecting to the Helios collective database.  Normally, this parameter should 
+not be necessary but is made available for users who may need to specify 
+special parameters in their database connections anyway.
+
+=head3 Other parameters
+
+=head4 pid_path
+
+ pid_path=/home/helios/run
+
+Sets the path where helios.pl daemon will write its PID files.  This should be 
+an absolute path to a directory writable by the Helios user (the user the 
+helios.pl daemon will run as).  Each helios.pl service daemon will write a PID 
+file incorporating the name of the service class it has loaded into this 
+directory.  
+
+If this directory does not exist or is not writable by the Helios user, the 
+helios.pl daemon will fail to start.
+
+DEFAULT:  /var/run/helios
+
+=head4 registration_interval
+
+The number of seconds to wait before a helios.pl service daemon "checks in" to 
+the collective database.  Periodically helios.pl will update a table in the 
+Helios collective database for monitoring purposes.  This allows the 
+Helios::Panoptes admin console to provide the Collective Admin view, and 
+enables Panoptes and other utilities to see if a helios.pl service daemon has 
+crashed or has encountered some other type of error.  The default 60 seconds 
+should be fine for most purposes, but can be increased to reduce database load 
+if necessary.
+
+DEFAULT:  60
+
+=head3 Service-specific Tuning Parameters for helios.pl
+
+There are several parameters useful for tuning the helios.pl service daemon to 
+work better with your Helios service.  Helios and the helios.pl daemon default 
+to behavior that should work well for processing jobs that last a short amount 
+of time (generally 30 seconds or less).  If your jobs consistently last longer
+than a minute, or can potentially put a strain on resources like a database or 
+a file server, you may wish to adjust the following parameters.
+
+These parameters are not dynamic and should be set in the Helios conf file, 
+either in the [global] section or a section matching your service's name.  
+
+=head4 master_launch_interval
+
+ master_launch_interval=5
+
+The master_launch_interval is the amount of time in seconds helios.pl waits 
+after launching workers before it attempts to launch workers again.  Normally 
+the default of 1 second is fine, but if you need to slow how quickly new worker 
+processes are started, you can increase this number.
+
+DEFAULT:  1
+
+=head4 zero_launch_interval
+
+ zero_launch_interval=30
+
+The zero_launch_interval is the amount of time in seconds helios.pl waits 
+to launch workers again after the MAX_WORKERS limit has been reached.  Once 
+helios.pl launches the MAX_WORKERS number of workers, it will not launch more 
+even if there are available jobs in the queue.  If a particular service's jobs 
+usually take longer than the default of 10 seconds, or you are using OVERDRIVE 
+mode so your worker processes persist until no more jobs available, increasing 
+zero_launch_interval may decrease needless database queries.  For most cases, 
+the default of 10 seconds should be adequate.
+
+DEFAULT:  10
+
+=head4 zero_sleep_interval
+
+ zero_sleep_interval=20
+
+The zero_sleep_interval is the amount of time between checks for available 
+jobs in the job queue when the job queue is empty.  If the helios.pl daemon 
+determines there are no available jobs for a service, it sleeps 
+zero_sleep_interval seconds and then checks again.  If there are available 
+jobs, it starts to launch workers; if there are still none, it sleeps another 
+zero_sleep_interval seconds and checks again.  This can cause jobs to "sit" in 
+the queue for some seconds before workers are launched to service them.  If 
+you do not have enough jobs consistently entering the job queue to keep workers 
+running in OVERDRIVE mode, decreasing this number will make helios.pl more 
+responsive by launching workers for your jobs sooner (at the expense of extra 
+repeated queries of the job queue in the database).  If your jobs can wait in 
+the job queue for awhile and you do not have many entering the system, 
+increasing this number can reduce the number of needless queries to your 
+database.
+
+DEFAULT:  10
+
+=head2 Worker process management
+
+The following configuration parameters affect the management of workers and 
+how they run services and process jobs.  These are most typically set in the 
+collective database configuration table for each service, thus they are ALL 
+CAPS to separate them from your services' own configuration parameters.  Unlike
+the parameters in the previous section, these configuration parameters are 
+dynamic and can be changed via Helios::Panoptes or SQL commands to your 
+collective database.
+
+=head3 HOLD
+
+ HOLD=1
+
+Puts a Helios service in Hold Mode.  All worker processes shut down after 
+finishing the current job, and the helios.pl service daemon ignores avaliable 
+jobs in the job queue.  Set HOLD to 0 or delete it from the configuration to 
+cause Helios to return to Normal Mode.
+
+DEFAULT:  0
+
+=head3 HALT
+
+ HALT=1
+
+Causes a helios.pl service daemon and all its workers to shutdown.  When HALT 
+is set for a service, worker processes exit after the current job is finished.  
+The helios.pl service daemon waits MAX_WORKER_TTL_WAIT_INTERVAL seconds for 
+workers to finish, and sends any remaining workers a SIGKILL signal to 
+eliminate any stragglers.  The daemon then removes its registration entry from 
+the collective database and exits.
+
+Warning:  BE CAREFUL about setting a HALT for a service for all hosts 
+(hostname='*').  This will shutdown all instances of that Helios service in 
+the ENTIRE collective, and the only way to restart them is to log into the 
+host and start them manually.  If you need to perform maintenance on hosts in 
+a production Helios collective, you most likely want to HOLD all instances of 
+a service and then HALT each instance individually as needed. 
+
+DEFAULT:  none (the presence of HALT in the config causes a shutdown regardless
+of its value)
+
+=head3 MAX_WORKERS
+
+ MAX_WORKERS=10
+
+Along with OVERDRIVE, MAX_WORKERS is the most powerful configuration 
+parameter in the Helios framework.  Setting MAX_WORKERS allows a helios.pl 
+service daemon to launch multiple workers at a time to service jobs, up to the 
+MAX_WORKERS limit.  
+
+Normally, when the helios.pl service daemon sees available jobs in the job 
+queue, it starts to launch worker processes to service the jobs.  Normally, 
+it launches workers gradually, one at a time, in order to prevent overtaxing 
+resources (and to allow the launched workers time to do actually run the 
+jobs).  If there are the same or more jobs in the queue as the MAX_WORKERS 
+value, helios.pl will "blitz" (launch the maximum amount of workers) to attempt 
+to run the jobs in the queue as quickly as possible.  This "blitzing" feature 
+is controlled by the WORKER_BLITZ_FACTOR parameter, so if you want want Helios 
+to blitz workers B<before> there are that many jobs available in the queue, 
+adjust WORKER_BLITZ_FACTOR downward to allow helios.pl to launch more worker 
+processes faster.
+
+DEFAULT:  1
+
+=head3 OVERDRIVE
+
+ OVERDRIVE=1
+
+Setting OVERDRIVE causes a worker process to persist in memory continuing to 
+run jobs from the job queue until all available jobs for the loaded service 
+are exhausted.  Coupled with MAX_WORKERS, allows you to maximize job 
+throughput by eliminating repeated process startup procedures and enabling 
+caching of database connections and other data structures.
+
+Unless your service is designed to run long-running jobs lazily, you almost 
+certainly want to set OVERDRIVE to 1.  It is set to 0 by default because 
+indiscriminately running untested, potentially unsafe services can cause 
+unexpected, even disasterous behavior.  Make sure your service runs in Normal 
+Mode first, then test it in Overdrive Mode throughly before you deploy it.
+
+DEFAULT:  0
+
+=head3 WORKER_BLITZ_FACTOR
+
+ WORKER_BLITZ_FACTOR=0.5
+
+Use WORKER_BLITZ_FACTOR to set the helios.pl service daemon to launch more 
+worker processes sooner.  Normally, helios.pl launches one worker process at a 
+time unless there are as many jobs as the MAX_WORKERS limit; if there are that 
+many jobs in the queue, Helios "blitzes" (launches MAX_WORKERS number of 
+workers).  If you want Helios to blitz workers sooner, adjust 
+WORKER_BLITZ_FACTOR lower.
+
+The WORKER_BLITZ_FACTOR is a decimal between 0 and 1.  If your service's 
+MAX_WORKERS parameter is set to 50 but you want Helios to launch the 
+maximum number of workers anytime there are at least 25 available jobs in the 
+job queue, you should set WORKER_BLITZ_FACTOR to 0.5.    
+
+DEFAULT:  1 (workers do not blitz until there are at least MAX_WORKERS jobs 
+available in the job queue)
+
+=head3 LAZY_CONFIG_UPDATE
+
+ LAZY_CONFIG_UPDATE=1
+
+Use LAZY_CONFIG_UPDATE to increase worker process performance by reducing the 
+number of configuration parameter refreshes a worker process performs in 
+Overdrive Mode.  In Overdrive Mode, a worker process refreshes the service 
+configuration from the collective database just before it calls the service's 
+run() method.  With LAZY_CONFIG_UPDATE set to 1, this configuration refresh is 
+performed only before every 10th job the worker process runs, reducing 
+database queries and thus increasing performance.
+
+NOTE:  The configuration refresh is where worker processes pick up HOLD and 
+HALT parameters, so using LAZY_CONFIG_UPDATE will cause worker processes to be 
+less responsive when holding jobs or halting the service.  If your service's 
+configuration does not change often, you can activate LAZY_CONFIG_UPDATE and 
+see if your service experiences a noticable increase. 
+
+DEFAULT:  0
+
+=head3 WORKER_MAX_TTL
+
+ WORKER_MAX_TTL=300
+
+The maximum amount of time in seconds to allow a worker process for a service 
+to run.  If a worker process continues to run past this threshold, the 
+helios.pl service daemon will assume it has become stuck in some way and will 
+send it a SIGKILL signal (9) to kill it (real world situations have shown 
+softer signals are unreliable in such situations).  If you set this and find 
+worker processes not experiencing problems are being unnecessarily killed, you 
+may need to increase the WORKER_MAX_TTL_WAIT_INTERVAL (below).
+
+DEFAULT:  none; workers running in Normal Mode run until their job is complete; 
+workers in Overdrive Mode work until no more jobs are available in the job 
+queue.
+
+=head3 WORKER_MAX_TTL_WAIT_INTERVAL
+
+Number of seconds a helios.pl service daemon will wait for a worker that has 
+reached its WORKER_MAX_TTL to exit.  If a worker process continues running 
+past WORKER_MAX_TTL + WORKER_MAX_TTL_WAIT_INTERVAL seconds, helios.pl will 
+assume the worker process has hung in some way and will send it a SIGKILL (9) 
+signal to kill it.
+
+DEFAULT:  30
+
+=head3 DOWNSHIFT_ON_NONZERO_RUN
+
+ DOWNSHIFT_ON_NONZERO_RUN=1
+
+This to support certain legacy behaviors for Helios services developed before 
+Helios 2.40.  You almost certainly do not need to set this.
+
+DEFAULT:  0 (ignore the return value of the service's run() method)
+
+=head2 Logging
+
+=head3 loggers
+
+ loggers=HeliosX::Logger::Syslog,HeliosX::Logger::Log4perl
+
+Specify a comma-separated list of external logging classes to use to log 
+information.  Each of the modules listed should implement the Helios::Logger 
+interface class.  
+
+Each logger class likely will have its own configuration parameters; see the 
+logger's documentation for the appropriate configuration information.
+
+The Helios internal logger (Helios::Logger::Internal) is automatically added 
+to this list, unless internal_logger (below) is turned off.
+
+DEFAULT:  None 
+
+=head3 internal_logger
+
+ internal_logger=off
+
+Whether the Helios internal logger (Helios::Logger::Internal) should be used 
+to log information.  The internal logger logs information to a table in the 
+Helios collective database, and is the log system used by the Helios::Panoptes 
+System Log view.  If you want to only use an external logging system such as 
+HeliosX::Logger::Log4perl, you can turn off Helios logging completely by 
+setting internal_logger to 0 or 'off'.
+
+DEFAULT:  on
+
+=head3 log_priority_threshold
+
+ log_priority_threshold=5
+
+The log level above which the internal logger discards log messages.  
+Specifying a log_priority_threshold will cause log messages of a lower priority 
+(higher numeric value) to be discarded.  For example, a log_priority_threshold
+of 6 (LOG_INFO) will cause log messages with a priority of 7 (LOG_DEBUG) to be 
+discarded.
+
+See the Helios::Logger::Internal documentation for more information on log 
+thresholds.
+
+DEFAULT:  undefined (all log messages are logged)
 
 =head1 AUTHOR
 
@@ -874,7 +961,7 @@ Andrew Johnson, E<lt>lajandy at cpan dot orgE<gt>
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (C) 2012-4 by Logical Helion, LLC.
+Copyright (C) 2012 by Logical Helion, LLC.
 
 This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself, either Perl version 5.8.0 or,
