@@ -1,11 +1,8 @@
 package Helios::Error::BaseError;
 
-use 5.008;
-use strict;
-use warnings;
 use base qw(Error::Simple);
 
-our $VERSION = '2.811_4150';
+our $VERSION = '2.40';
 
 1;
 __END__
