@@ -1,11 +1,8 @@
 package Helios::Error::ConfigError;
 
-use 5.008;
-use strict;
-use warnings;
 use base qw(Helios::Error::BaseError);
 
-our $VERSION = '2.811_4150';
+our $VERSION = '2.60';
 
 1;
 __END__;
